from __future__ import annotations

from typing import Union, List, Tuple, Sequence, Dict

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .MeshNode import MeshNode


@abaqus_class_doc
class MeshNodeArray(List[MeshNode]):
    """The MeshNodeArray is a sequence of MeshNode objects.

    .. note::
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].allInternalSets[name].nodes
            mdb.models[name].parts[name].allInternalSurfaces[name].nodes
            mdb.models[name].parts[name].allSets[name].nodes
            mdb.models[name].parts[name].allSurfaces[name].nodes
            mdb.models[name].parts[name].nodes
            mdb.models[name].parts[name].retainedNodes
            mdb.models[name].parts[name].sets[name].nodes
            mdb.models[name].parts[name].surfaces[name].nodes
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].nodes
            mdb.models[name].rootAssembly.allInstances[name].sets[name].nodes
            mdb.models[name].rootAssembly.allInstances[name].surfaces[name].nodes
            mdb.models[name].rootAssembly.allInternalSets[name].nodes
            mdb.models[name].rootAssembly.allInternalSurfaces[name].nodes
            mdb.models[name].rootAssembly.allSets[name].nodes
            mdb.models[name].rootAssembly.allSurfaces[name].nodes
            mdb.models[name].rootAssembly.instances[name].nodes
            mdb.models[name].rootAssembly.instances[name].sets[name].nodes
            mdb.models[name].rootAssembly.instances[name].surfaces[name].nodes
            mdb.models[name].rootAssembly.modelInstances[i].nodes
            mdb.models[name].rootAssembly.modelInstances[i].sets[name].nodes
            mdb.models[name].rootAssembly.modelInstances[i].surfaces[name].nodes
            mdb.models[name].rootAssembly.nodes
            mdb.models[name].rootAssembly.sets[name].nodes
            mdb.models[name].rootAssembly.surfaces[name].nodes
    """

    @abaqus_method_doc
    def __init__(self, nodes: List[MeshNode]) -> None:
        """This method creates a MeshNodeArray object.

        .. note::
            This function can be accessed by::

                mesh.MeshNodeArray

        Parameters
        ----------
        nodes
            A list of MeshNode objects.

        Returns
        -------
        MeshNodeArray
            A MeshNodeArray object.
        """
        super().__init__()

    @abaqus_method_doc
    def getFromLabel(self, label: int) -> MeshNode:
        """This method returns the object in the MeshNodeArray with the given label.

        Parameters
        ----------
        label
            An Int specifying the label of the object.

        Returns
        -------
        MeshNode
            A MeshNode object.
        """
        ...

    @abaqus_method_doc
    def getSequenceFromMask(self, mask: Union[str, Sequence[str]]) -> MeshNodeArray:
        """This method returns the objects in the MeshNodeArray identified using the specified
        **mask**. This command is generated when the JournalOptions are set to COMPRESSEDINDEX.
        When a large number of objects are involved, this method is highly efficient.

        Parameters
        ----------
        mask
            A String specifying the object or objects.

        Returns
        -------
        MeshNodeArray
            A MeshNodeArray object.
        """
        ...

    @abaqus_method_doc
    def getMask(self) -> str:
        """This method returns a string specifying the object or objects.

        Returns
        -------
        str
            A String specifying the object or objects.
        """
        ...

    @abaqus_method_doc
    def getByBoundingBox(
        self,
        xMin: float = ...,
        yMin: float = ...,
        zMin: float = ...,
        xMax: float = ...,
        yMax: float = ...,
        zMax: float = ...,
    ) -> MeshNodeArray:
        """This method returns an array of nodes that lie within the specified bounding box.

        Parameters
        ----------
        xMin
            A float specifying the minimum X boundary of the bounding box.
        yMin
            A float specifying the minimum Y boundary of the bounding box.
        zMin
            A float specifying the minimum Z boundary of the bounding box.
        xMax
            A float specifying the maximum X boundary of the bounding box.
        yMax
            A float specifying the maximum Y boundary of the bounding box.
        zMax
            A float specifying the maximum Z boundary of the bounding box.

        Returns
        -------
        MeshNodeArray
            A MeshNodeArray object, which is a sequence of MeshNode objects.
        """
        ...

    @abaqus_method_doc
    def getByBoundingCylinder(
        self,
        center1: Tuple[float, float, float],
        center2: Tuple[float, float, float],
        radius: float,
    ) -> MeshNodeArray:
        """This method returns an array of node objects that lie within the specified bounding
        cylinder.

        Parameters
        ----------
        center1
            A tuple of the X-, Y-, and Z-coordinates of the center of the first end of the cylinder.
        center2
            A tuple of the X-, Y-, and Z-coordinates of the center of the second end of the
            cylinder.
        radius
            A float specifying the radius of the cylinder.

        Returns
        -------
        MeshNodeArray
            A MeshNodeArray object, which is a sequence of MeshNode objects.
        """
        ...

    @abaqus_method_doc
    def getByBoundingSphere(self, center: tuple, radius: str) -> MeshNodeArray:
        """This method returns an array of node objects that lie within the specified bounding
        sphere.

        Parameters
        ----------
        center
            A tuple of the X-, Y-, and Z-coordinates of the center of the sphere.
        radius
            A float specifying the radius of the sphere.

        Returns
        -------
        MeshNodeArray
            A MeshNodeArray object, which is a sequence of MeshNode objects.
        """
        ...

    @abaqus_method_doc
    def getBoundingBox(self) -> Dict[str, Tuple[float, float, float]]:
        """This method returns a dictionary of two tuples representing minimum and maximum boundary
        values of the bounding box of the minimum size containing the node sequence.

        Returns
        -------
        Dict[str, Tuple[float, float, float]]
            A Dictionary object with the following items:

            - **low**: a tuple of three floats representing the minimum x, y and z boundary values of
              the bounding box.
            - **high**: a tuple of three floats representing the maximum x, y and z boundary values of
              the bounding box.

        Raises
        ------
        """
        ...

    @abaqus_method_doc
    def getClosest(
        self, coordinates: str, numToFind: int = 1, searchTolerance: float = ...
    ) -> Union[MeshNode, List[MeshNode]]:
        """This method returns the node or nodes closest to the given point or set of points.

        Parameters
        ----------
        coordinates
            A point defined by x, y, and z values or a list of such points.
        numToFind
            The number of nodes to find for each given point. For example, if **numToFind** is 2, then
            the 2 closest points, if available and within **searchTolerance**, will be returned in
            order of proximity for each input point. The default is 1.
        searchTolerance
            A float specifying a search radius for each point. By default, no search radius is
            defined, and all nodes in the sequence will be searched.

        Returns
        -------
        MeshNode | List[MeshNode]
            A MeshNode, or a list of MeshNode objects, or a list of lists of MeshNode objects,
            depending on the number of points given and the number of nodes requested.
        """
        ...

    @abaqus_method_doc
    def sequenceFromLabels(self, labels: Sequence[int]) -> MeshNodeArray:
        """This method returns the objects in the MeshNodeArray identified using the specified
        labels.

        Parameters
        ----------
        labels
            A sequence of Ints specifying the labels.

        Returns
        -------
        MeshNodeArray
            A MeshNodeArray object.

        Raises
        ------
        Error: The mask results in an empty sequence
            An exception occurs if the resulting sequence is empty.
        """
        ...



from typing import Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ...UtilityAndView.abaqusConstants import Boolean, OFF
from ...UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class PlasticityCorrection:
    r"""The PlasticityCorrection object specifies the elastic-plastic response of the material with linear elasticity.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].plasticityCorrection

        The table data for this object are:

        - If type=RAMBERG_OSGOOD, the table specifies the following:

            - :math:`K`.
            - :math:`n`.
            - Temperature if the data depend on temperature.
            - Value of the first field variable if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        - If type=TABULAR, the table specifies the following:

            - Yield stress.
            - Plastic strain.
            - Temperature if the data depend on temperature.
            - Value of the first field variable if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        The corresponding analysis keywords are:

        - PLASTICITY CORRECTION

    .. versionadded:: 2023

        The `PlasticityCorrection` class was added.
    """

    #: Set type=RAMBERG_OSGOOD to specify the Ramberg-Osgood relationship.
    #: Set type=TABULAR to specify the tabular form.
    type: Literal[C.RAMBERG_OSGOOD, C.TABULAR]

    #: A sequence of sequences of Floats specifying the items described below.
    table: Sequence[Sequence[float]]

    #: A Boolean specifying whether the data depend on temperature. The default value is OFF.
    temperatureDependency: Boolean = OFF

    #: An Int specifying the number of field variable dependencies. The default value is 0.
    dependencies: int = 0

    @abaqus_method_doc
    def __init__(
        self,
        type: Literal[C.RAMBERG_OSGOOD, C.TABULAR],
        table: Sequence[Sequence[float]],
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
    ):
        """This method creates a PlasticityCorrection object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].PlasticityCorrection
                session.odbs[name].materials[name].PlasticityCorrection

        Parameters
        ----------
        type
            Set type=RAMBERG_OSGOOD to specify the Ramberg-Osgood relationship.
            Set type=TABULAR to specify the tabular form.
        table
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        PlasticityCorrection
            A PlasticityCorrection object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the PlasticityCorrection object.

        Raises
        ------
        RangeError
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .CombinedTestData import CombinedTestData
from ....Others.Mechanical.Viscosity.Trs import Trs
from ....TestData.ShearTestData import ShearTestData
from ....TestData.VolumetricTestData import VolumetricTestData
from .....UtilityAndView.abaqusConstants import FORMULA, ISOTROPIC, NONE, PRONY
from .....UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class Viscoelastic:
    r"""The Viscoelastic object specifies dissipative behavior for use with elasticity.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].viscoelastic
            import odbMaterial
            session.odbs[name].materials[name].viscoelastic

        The table data for this object are:

        - If **frequency** = FORMULA, the table data for **table** specify the following:

            - Real part of :math:`g_{1}^{*}\left(g^{*}(\omega)=g_{1}^{*} f^{-a}\right)`
            - Imaginary part of :math:`g_{1}^{*}`.
            - Value of :math:`a`.
            - Real part of :math:`k_{1}^{*}\left(k^{*}(\omega)=k_{1}^{*} f^{-b}\right)`. If
              the material is incompressible, this value is ignored.
            - Imaginary part of :math:`k_{1}^{*}`. If the material is incompressible, this
              value is ignored.
            - Value of :math:`b`. If the material is incompressible, this value is ignored.

        - If **frequency** = TABULAR and **type** = ISOTROPIC and **preload** = NONE, or **time** = FREQUENCY_DATA
          the table data for **table** specify the following:

            - Real part of :math:`\omega g^{*}\left(\omega \Re\left(g^{*}\right)=G_{\ell} / G_{\infty}\right)`.
            - Imaginary part of :math:`\omega g^{*}\left(\omega \mathfrak{J}\left(g^{*}\right)=1-G_{s} / G_{\infty}\right)`
            - Real part of :math:`\omega k^{*}\left(\omega \mathfrak{R}\left(k^{*}\right)=\mathrm{K}_{\ell} / \mathrm{K}_{\infty}\right)`. If the material is incompressible, this value is ignored.
            - Imaginary part of :math:`\omega k^{*}\left(\omega \mathfrak{I}\left(k^{*}\right)=1-\mathrm{K}_{s} / \mathrm{K}_{\infty}\right)`. If the material is incompressible, this value is ignored.
            - Frequency :math:`f` in cycles per time.

        - If **frequency** = TABULAR and **type** = ISOTROPIC and **preload** = UNIAXIAL the table data for **table**
          specify the following:

            - Loss modulus.
            - Storage modulus.
            - Frequency.
            - Uniaxial strain.

        - If **frequency** = TABULAR and **type** = TRACTION and **preload** = NONE the table data for **table**
          specify the following:

            - Normalized loss modulus.
            - Normalized shear modulus.
            - Frequency.

        - If **frequency** = TABULAR and **type** = TRACTION and **preload** = UNIAXIAL or
          **preload** = UNIAXIAL_VOLUMETRIC the table data for **table** specify the following:

            - Loss modulus.
            - Storage modulus.
            - Frequency.
            - Closure.

        - If **time** = PRONY or **frequency** = PRONY, the table data for **table** specify the following:

            - :math:`\bar{g}_{1}^{P}`, the modulus ratio in the first term in the Prony series
              expansion of the shear relaxation modulus.
            - :math:`\bar{k}_{1}^{P}`, the modulus ratio in the first term in the Prony series
              expansion of the bulk relaxation modulus.
            - :math:`\tau_{1}`, the relaxation time for the first term in the Prony series expansion.

        - If **frequency** = TABULAR and **type** = ISOTROPIC and **preload** = VOLUMETRIC or
          **preload** = UNIAXIAL_VOLUMETRIC the table data for **volumetricTable** specify the following:

            - Loss modulus.
            - Storage modulus.
            - Frequency.
            - Volume ratio.

        The corresponding analysis keywords are:

        - VISCOELASTIC
    """

    #: A CombinedTestData object.
    combinedTestData: CombinedTestData = CombinedTestData(((),))

    #: A ShearTestData object.
    shearTestData: ShearTestData = ShearTestData(((),))

    #: A Trs object.
    trs: Trs = Trs()

    #: A VolumetricTestData object.
    volumetricTestData: VolumetricTestData = VolumetricTestData(((),))

    @abaqus_method_doc
    def __init__(
        self,
        domain: Literal[C.FREQUENCY, C.TIME],
        table: tuple,
        frequency: Literal[
            C.PRONY, C.CREEP_TEST_DATA, C.TABULAR, C.FREQUENCY, C.RELAXATION_TEST_DATA, C.FORMULA
        ] = FORMULA,
        type: Literal[C.FREQUENCY, C.TRACTION, C.ISOTROPIC, C.TABULAR] = ISOTROPIC,
        preload: Literal[C.VOLUMETRIC, C.UNIAXIAL, C.TABULAR, C.FREQUENCY, C.UNIAXIAL_VOLUMETRIC, C.NONE] = NONE,
        time: Literal[C.PRONY, C.CREEP_TEST_DATA, C.FREQUENCY_DATA, C.TIME, C.RELAXATION_TEST_DATA] = PRONY,
        errtol: float = 0,
        nmax: int = 13,
        volumetricTable: tuple = (),
    ):
        """This method creates a Viscoelastic object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].Viscoelastic
                session.odbs[name].materials[name].Viscoelastic

        Parameters
        ----------
        domain
            A SymbolicConstant specifying the domain definition. Possible values are:
            - FREQUENCY, specifying a frequency domain. This domain is only available for an Abaqus/Standard analysis.
            - TIME, specifying a time domain.
        table
            A sequence of sequences of Floats specifying the items described below.
        frequency
            A SymbolicConstant specifying the frequency domain definition. This argument is required
            only when **domain** = FREQUENCY. Possible values are FORMULA, TABULAR, PRONY,
            CREEP_TEST_DATA, and RELAXATION_TEST_DATA. The default value is FORMULA.
        type
            A SymbolicConstant specifying the type. This argument is required only when
            **domain** = FREQUENCY and **frequency** = TABULAR. Possible values are ISOTROPIC and TRACTION.
            The default value is ISOTROPIC.
        preload
            A SymbolicConstant specifying the preload. This argument is required only when
            **domain** = FREQUENCY and **frequency** = TABULAR. Possible values are NONE, UNIAXIAL,
            VOLUMETRIC, and UNIAXIAL_VOLUMETRIC. The default value is NONE.
        time
            A SymbolicConstant specifying the time domain definition. This argument is required only
            when **domain** = TIME. Possible values are PRONY, CREEP_TEST_DATA, RELAXATION_TEST_DATA,
            and FREQUENCY_DATA. The default value is PRONY.
        errtol
            A Float specifying the allowable average root-mean-square error of the data points in
            the least-squares fit. The Float values correspond to percentages; for example, 0.01 is
            1%. The default value is 0.01.This argument is valid only when **time** = CREEP_TEST_DATA,
            RELAXATION_TEST_DATA or FREQUENCY_DATA; or only when **frequency** = CREEP_TEST_DATA or
            RELAXATION_TEST_DATA.
        nmax
            An Int specifying the maximum number of terms NN in the Prony series. The maximum value
            is 13. The default value is 13.This argument is valid only when **time** = CREEP_TEST_DATA,
            RELAXATION_TEST_DATA or FREQUENCY_DATA; or only when **frequency** = CREEP_TEST_DATA or
            RELAXATION_TEST_DATA.
        volumetricTable
            A sequence of sequences of Floats specifying the items described below. The default
            value is an empty sequence.

        Returns
        -------
        Viscoelastic
            A Viscoelastic object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the Viscoelastic object.

        Raises
        ------
        RangeError
        """
        ...



from typing import Union

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from ..UtilityAndView.abaqusConstants import Boolean, DEFAULT, OFF, ON
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class Control:
    """The Control object is used to provide additional optional general solution controls.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].steps[name].control
    """

    @abaqus_method_doc
    def setValues(
        self,
        allowPropagation: Boolean = ON,
        resetDefaultValues: Boolean = OFF,
        discontinuous: Boolean = OFF,
        constraints: Union[Literal[C.DEFAULT], float] = DEFAULT,
        lineSearch: Union[Literal[C.DEFAULT], float] = DEFAULT,
        timeIncrementation: Union[Literal[C.DEFAULT], float] = DEFAULT,
        directCyclic: Union[Literal[C.DEFAULT], float] = DEFAULT,
        concentrationField: Union[Literal[C.DEFAULT], float] = DEFAULT,
        displacementField: Union[Literal[C.DEFAULT], float] = DEFAULT,
        electricalPotentialField: Union[Literal[C.DEFAULT], float] = DEFAULT,
        globalField: Union[Literal[C.DEFAULT], float] = DEFAULT,
        hydrostaticFluidPressureField: Union[Literal[C.DEFAULT], float] = DEFAULT,
        poreFluidPressureField: Union[Literal[C.DEFAULT], float] = DEFAULT,
        rotationField: Union[Literal[C.DEFAULT], float] = DEFAULT,
        temperatureField: Union[Literal[C.DEFAULT], float] = DEFAULT,
        vcctLinearScaling: Union[Literal[C.DEFAULT, C.VCCT], float] = DEFAULT,
    ):
        """This method modifies the Control object.

        Parameters
        ----------
        allowPropagation
            A Boolean specifying whether to allow all control values to propagate from a previous
            step. Setting this argument to ON automatically sets **resetDefaultValues** to OFF. The
            default value is ON.
        resetDefaultValues
            A Boolean specifying whether to use all default control values. Setting this argument to
            ON automatically sets **allowPropagation** to OFF. The default value is OFF.
        discontinuous
            A Boolean specifying whether to set **timeIncrementation** values that will usually
            improve efficiency for analyses with severely discontinuous behavior. The default value
            is OFF.
        constraints
            The SymbolicConstant DEFAULT or a sequence of Floats specifying tolerances on constraint
            equations. If a specified sequence contains a value of 0, that item in the sequence will
            be set to its system-defined value. The value can also be the SymbolicConstant DEFAULT.
            The default value is DEFAULT.
        lineSearch
            The SymbolicConstant DEFAULT or a sequence of Floats specifying line search control
            parameters. If a specified sequence contains a value of 0, that item in the sequence
            will be set to its system-defined value. The value can also be the SymbolicConstant
            DEFAULT. The default value is DEFAULT.
        timeIncrementation
            The SymbolicConstant DEFAULT or a sequence of Floats specifying time incrementation
            control parameters. If a specified sequence contains a value of 0, that item in the
            sequence will be set to its system-defined value. The value can also be the
            SymbolicConstant DEFAULT. The default value is DEFAULT.
        directCyclic
            The SymbolicConstant DEFAULT or a sequence of Floats specifying direct cyclic control
            parameters. If a specified sequence contains a value of 0, that item in the sequence
            will be set to its system-defined value. The value can also be the SymbolicConstant
            DEFAULT. The default value is DEFAULT.
        concentrationField
            The SymbolicConstant DEFAULT or a sequence of Floats specifying mass concentration field
            equilibrium equation parameters. If a specified sequence contains a value of 0, that
            item in the sequence will be set to its system-defined value. The value can also be the
            SymbolicConstant DEFAULT. The default value is DEFAULT.
        displacementField
            The SymbolicConstant DEFAULT or a sequence of Floats specifying displacement field and
            warping degree of freedom field equilibrium equation parameters. If a specified sequence
            contains a value of 0, that item in the sequence will be set to its system-defined
            value. The value can also be the SymbolicConstant DEFAULT. The default value is DEFAULT.
        electricalPotentialField
            The SymbolicConstant DEFAULT or a sequence of Floats specifying electrical potential
            field equilibrium equation parameters. If a specified sequence contains a value of 0,
            that item in the sequence will be set to its system-defined value. The value can also be
            the SymbolicConstant DEFAULT. The default value is DEFAULT.
        globalField
            The SymbolicConstant DEFAULT or a sequence of Floats specifying parameters for all
            applicable field equilibrium equations. This argument overwrites all other field
            equilibrium equation control values. If a specified sequence contains a value of 0, that
            item in the sequence will be set to its system-defined value. The value can also be the
            SymbolicConstant DEFAULT. The default value is DEFAULT.
        hydrostaticFluidPressureField
            The SymbolicConstant DEFAULT or a sequence of Floats specifying hydrostatic fluid
            element volume constraint parameters. If a specified sequence contains a value of 0,
            that item in the sequence will be set to its system-defined value. The value can also be
            the SymbolicConstant DEFAULT. The default value is DEFAULT.
        poreFluidPressureField
            The SymbolicConstant DEFAULT or a sequence of Floats specifying pore liquid volumetric
            continuity equilibrium equation parameters. If a specified sequence contains a value of
            0, that item in the sequence will be set to its system-defined value. The value can also
            be the SymbolicConstant DEFAULT. The default value is DEFAULT.
        rotationField
            The SymbolicConstant DEFAULT or a sequence of Floats specifying rotation field
            equilibrium equation parameters. If a specified sequence contains a value of 0, that
            item in the sequence will be set to its system-defined value. The value can also be the
            SymbolicConstant DEFAULT. The default value is DEFAULT.
        temperatureField
            The SymbolicConstant DEFAULT or a sequence of Floats specifying temperature field
            equilibrium equation parameters. If a specified sequence contains a value of 0, that
            item in the sequence will be set to its system-defined value. The value can also be the
            SymbolicConstant DEFAULT. The default value is DEFAULT.
        vcctLinearScaling
            The SymbolicConstant DEFAULT or a Float specifying linear scaling parameter for a VCCT
            debonding analysis. If a specified value is 0, it will be set to its system-defined
            value. The value can also be the SymbolicConstant DEFAULT. The default value is DEFAULT.

        Raises
        ------
        RangeError
        """
        ...



from abqpy.decorators import abaqus_class_doc

from .Section import Section


@abaqus_class_doc
class SolidSection(Section):
    """The ShellSection object defines the properties of a shell section. The ShellSection
    object is derived from the Section object. The ShellSection object has no explicit
    constructor and no methods or members.
    The ShellSection object is derived from the Section object.

    .. note::
        This object can be accessed by::

            import section
            mdb.models[name].sections[name]
            import odbSection
            session.odbs[name].sections[name]
    """

    #: A String specifying the repository key.
    name: str = ""



from typing import Optional

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class StructuralDampingByFrequencyComponent:
    """A StructuralDampingByFrequencyComponent object is used to define structural damping over
    a range of frequencies.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].steps[name].structuralDampingByFrequency.components[i]
    """

    #: A Float specifying the frequency value in cycles/time.
    frequency: Optional[float] = None

    #: A Float specifying the damping factor, s.
    factor: Optional[float] = None



from typing import Union, Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .CohesiveBehavior import CohesiveBehavior
from .ContactDamage import ContactDamage
from .ContactDamping import ContactDamping
from .ContactTangentialBehavior import ContactTangentialBehavior
from .FractureCriterion import FractureCriterion
from .GapElectricalConductance import GapElectricalConductance
from .GapHeatGeneration import GapHeatGeneration
from .GeometricProperties import GeometricProperties
from .InteractionProperty import InteractionProperty
from .NormalBehavior import NormalBehavior
from .Radiation import Radiation
from .ThermalConductance import ThermalConductance
from ..UtilityAndView.abaqusConstants import (
    ALL_NODES,
    BK,
    Boolean,
    COEFFICIENTS,
    DAMPING_COEFFICIENT,
    DEFAULT,
    DISPLACEMENT,
    ENERGY,
    FRACTION,
    FRICTIONLESS,
    HARD,
    ISOTROPIC,
    LINEAR,
    MAX_STRESS,
    OFF,
    ON,
    STEP,
    SymbolicConstant,
    TABULAR,
    UNCOUPLED,
    VCCT,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class ContactProperty(InteractionProperty):
    """The ContactProperty object defines a contact interaction property.
    The ContactProperty object is derived from the InteractionProperty object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactionProperties[name]

        The corresponding analysis keywords are:

        - SURFACE INTERACTION
    """

    #: A ContactTangentialBehavior object.
    tangentialBehavior: ContactTangentialBehavior = ContactTangentialBehavior()

    #: A NormalBehavior object.
    normalBehavior: NormalBehavior = NormalBehavior()

    #: A ContactDamping object.
    damping: ContactDamping = ContactDamping()

    #: A ContactDamage object.
    damage: ContactDamage = ContactDamage(((),))

    #: A FractureCriterion object.
    fractureCriterion: FractureCriterion = FractureCriterion(((),))

    #: A CohesiveBehavior object.
    cohesiveBehavior: CohesiveBehavior = CohesiveBehavior()

    #: A ThermalConductance object.
    thermalConductance: ThermalConductance = ThermalConductance()

    #: A GapHeatGeneration object.
    heatGeneration: GapHeatGeneration = GapHeatGeneration()

    #: A Radiation object.
    radiation: Optional[Radiation] = None

    #: A GeometricProperties object.
    geometricProperties: GeometricProperties = GeometricProperties()

    #: A GapElectricalConductance object.
    electricalConductance: GapElectricalConductance = GapElectricalConductance()

    @abaqus_method_doc
    def __init__(self, name: str):
        """This method creates a ContactProperty object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.

        Returns
        -------
        ContactProperty
            A ContactProperty object.
        """
        super().__init__()

    @abaqus_method_doc
    def TangentialBehavior(
        self,
        formulation: Literal[
            C.FRICTIONLESS, C.PENALTY, C.ROUGH, C.EXPONENTIAL_DECAY, C.USER_DEFINED, C.LAGRANGE
        ] = FRICTIONLESS,
        directionality: Literal[C.ANISOTROPIC, C.ISOTROPIC] = ISOTROPIC,
        slipRateDependency: Boolean = OFF,
        pressureDependency: Boolean = OFF,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        exponentialDecayDefinition: Literal[C.TEST_DATA, C.COEFFICIENTS] = COEFFICIENTS,
        table: tuple = (),
        shearStressLimit: Optional[float] = None,
        maximumElasticSlip: Literal[C.FRACTION, C.ABSOLUTE_DISTANCE] = FRACTION,
        fraction: float = 0,
        absoluteDistance: float = 0,
        elasticSlipStiffness: Optional[float] = None,
        nStateDependentVars: int = 0,
        useProperties: Boolean = OFF,
    ):
        """This method creates a ContactTangentialBehavior object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        formulation
            A SymbolicConstant specifying the friction formulation. Possible values are
            FRICTIONLESS, PENALTY, EXPONENTIAL_DECAY, ROUGH, LAGRANGE, and USER_DEFINED. The default
            value is FRICTIONLESS.
        directionality
            A SymbolicConstant specifying the directionality of the friction. Possible values are
            ISOTROPIC and ANISOTROPIC. The default value is ISOTROPIC.
        slipRateDependency
            A Boolean specifying whether the data depend on slip rate. The default value is OFF.
        pressureDependency
            A Boolean specifying whether the data depend on contact pressure. The default value is
            OFF.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variables. The default value is 0.
        exponentialDecayDefinition
            A SymbolicConstant specifying the exponential decay definition. Possible values are
            COEFFICIENTS and TEST_DATA. The default value is COEFFICIENTS.
        table
            A sequence of sequences of Floats specifying tangential behavior. The items in the table
            data are described below.
        shearStressLimit
            None or a Float specifying the shear stress limit. If **shearStressLimit** = None, there is
            no upper limit. The default value is None.
        maximumElasticSlip
            A SymbolicConstant specifying what the maximum elastic slip will be. Possible values are
            FRACTION and ABSOLUTE_DISTANCE. The default value is FRACTION.
        fraction
            A Float specifying the fraction of a characteristic surface dimension. The default value
            is 0.0.
        absoluteDistance
            A Float specifying the absolute distance. The default value is 0.0.
        elasticSlipStiffness
            None or a Float specifying the elastic slip stiffness. If **elasticSlipStiffness** = None,
            there is no upper limit. The default value is None.
        nStateDependentVars
            An Int specifying the number of state-dependent variables. The default value is 0.
        useProperties
            A Boolean specifying whether property values will be used. The default value is OFF.

        Returns
        -------
        ContactTangentialBehavior
            A ContactTangentialBehavior object.
        """
        self.tangentialBehavior = ContactTangentialBehavior(
            formulation,
            directionality,
            slipRateDependency,
            pressureDependency,
            temperatureDependency,
            dependencies,
            exponentialDecayDefinition,
            table,
            shearStressLimit,
            maximumElasticSlip,
            fraction,
            absoluteDistance,
            elasticSlipStiffness,
            nStateDependentVars,
            useProperties,
        )
        return self.tangentialBehavior

    @abaqus_method_doc
    def NormalBehavior(
        self,
        contactStiffness: Union[Literal[C.AUGMENTED_LAGRANGE, C.PENALTY, C.DEFAULT, C.HARD, C.LINEAR], float] = DEFAULT,
        pressureOverclosure: Literal[C.EXPONENTIAL, C.TABULAR, C.HARD, C.LINEAR, C.SCALE_FACTOR] = HARD,
        allowSeparation: Boolean = ON,
        maxStiffness: Optional[float] = None,
        table: tuple = (),
        constraintEnforcementMethod: Literal[C.AUGMENTED_LAGRANGE, C.DEFAULT, C.DIRECT, C.PENALTY] = DEFAULT,
        overclosureFactor: float = 0,
        overclosureMeasure: float = 0,
        contactStiffnessScaleFactor: float = 1,
        initialStiffnessScaleFactor: float = 1,
        clearanceAtZeroContactPressure: float = 0,
        stiffnessBehavior: Literal[C.PENALTY, C.NONLINEAR, C.LINEAR] = LINEAR,
        stiffnessRatio: float = 0,
        upperQuadraticFactor: float = 0,
        lowerQuadraticRatio: float = 0,
    ):
        """This method creates a NormalBehavior object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        contactStiffness
            The SymbolicConstant DEFAULT or a Float specifying the contact stiffness. This argument
            is valid for **pressureOverclosure** = LINEAR. This argument is also valid for
            **pressureOverclosure** = HARD when **constraintEnforcementMethod** = AUGMENTED_LAGRANGE or
            PENALTY. A value of DEFAULT is valid only when the later conditions are met. A value of
            zero is equivalent to specifying DEFAULT. The default value is DEFAULT.
        pressureOverclosure
            A SymbolicConstant specifying the pressure-overclosure relationship to be used. Possible
            values are HARD, EXPONENTIAL, LINEAR, TABULAR, and SCALE_FACTOR. The default value is
            HARD.
        allowSeparation
            A Boolean specifying whether to allow separation after contact. The default value is ON.
        maxStiffness
            None or a Float specifying the maximum stiffness. If **maxStiffness** = None, there is no
            upper limit. The default value is None.
        table
            A sequence of sequences of Floats specifying the normal behavior properties. This
            argument is valid only for **pressureOverclosure** = EXPONENTIAL or TABULAR. The items in
            the table data are described below.
        constraintEnforcementMethod
            A SymbolicConstant specifying the method for enforcement of the contact constraint.
            Possible values are DEFAULT, AUGMENTED_LAGRANGE, PENALTY, and DIRECT. The default value
            is DEFAULT.
        overclosureFactor
            A Float specifying the overclosure measure (used to delineate the segments of the
            pressure-overclosure curve) as a percentage of the minimum element size in the contact
            region. The default value is 0.0.
        overclosureMeasure
            A Float specifying the overclosure measure (used to delineate the segments of the
            pressure-overclosure curve) directly. The default value is 0.0.
        contactStiffnessScaleFactor
            A Float specifying scale factor for the penalty stiffness or the geometric scaling of
            the "base" stiffness. The default value is 1.0.
        initialStiffnessScaleFactor
            A Float specifying an additional scale factor for the "base" default contact stiffness.
            The default value is 1.0.
        clearanceAtZeroContactPressure
            A Float specifying the clearance at which the contact pressure is zero. The default
            value is 0.0.
        stiffnessBehavior
            A SymbolicConstant specifying the type of penalty stiffness to be defined. This argument
            is valid only when **constraintEnforcementMethod** = PENALTY. Possible values are LINEAR and
            NONLINEAR. The default value is LINEAR.
        stiffnessRatio
            A Float specifying the ratio of the initial stiffness divided by the final stiffness.
            This argument is valid only when **stiffnessBehavior** = NONLINEAR. Possible values are 0 ≤
            **stiffnessRatio** < 1. The default value is 0.01.
        upperQuadraticFactor
            A Float specifying the ratio of the overclosure at the maximum stiffness divided by the
            characteristic facet length. This argument is valid only when
            **stiffnessBehavior** = NONLINEAR. The default value is 0.03.
        lowerQuadraticRatio
            A Float specifying the ratio of the overclosure at the initial stiffness divided by the
            overclosure at the maximum stiffness, both relative to the clearance at which the
            contact pressure is zero. This argument is valid only when
            **stiffnessBehavior** = NONLINEAR. Possible values are 0 ≤ **stiffnessRatio** < 1. The
            default value is 0.33333.

        Returns
        -------
        NormalBehavior
            A NormalBehavior object.
        """
        self.normalBehavior = NormalBehavior(
            contactStiffness,
            pressureOverclosure,
            allowSeparation,
            maxStiffness,
            table,
            constraintEnforcementMethod,
            overclosureFactor,
            overclosureMeasure,
            contactStiffnessScaleFactor,
            initialStiffnessScaleFactor,
            clearanceAtZeroContactPressure,
            stiffnessBehavior,
            stiffnessRatio,
            upperQuadraticFactor,
            lowerQuadraticRatio,
        )
        return self.normalBehavior

    @abaqus_method_doc
    def Damping(
        self,
        definition: Literal[C.DAMPING_COEFFICIENT, C.CRITICAL_DAMPING_FRACTION] = DAMPING_COEFFICIENT,
        tangentFraction: Union[Literal[C.DEFAULT], float] = DEFAULT,
        clearanceDependence: Literal[C.STEP, C.CRITICAL_DAMPING_FRACTION, C.BILINEAR, C.LINEAR] = STEP,
        table: tuple = (),
    ):
        """This method creates a ContactDamping object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        definition
            A SymbolicConstant specifying the method used to define the damping. Possible values are
            DAMPING_COEFFICIENT and CRITICAL_DAMPING_FRACTION. The default value is
            DAMPING_COEFFICIENT.
        tangentFraction
            The SymbolicConstant DEFAULT or a Float specifying the tangential damping coefficient
            divided by the normal damping coefficient. The default value is DEFAULT.
        clearanceDependence
            A SymbolicConstant specifying the variation of the damping coefficient or fraction with
            respect to clearance. Possible values are STEP, LINEAR, and BILINEAR. The default value
            is STEP.If **definition** = CRITICAL_DAMPING_FRACTION, the only possible value is STEP.
        table
            A sequence of pairs of Floats specifying the damping properties. The items in the table
            data are described below.

        Returns
        -------
        ContactDamping
            A ContactDamping object.
        """
        self.damping = ContactDamping(definition, tangentFraction, clearanceDependence, table)
        return self.damping

    @abaqus_method_doc
    def Damage(
        self,
        initTable: tuple,
        criterion: Literal[C.MAX_STRESS, C.QUAD_SEPARATION, C.MAX_SEPARATION, C.QUAD_TRACTION] = MAX_STRESS,
        initTempDep: Boolean = OFF,
        initDependencies: int = 0,
        useEvolution: Boolean = OFF,
        evolutionType: Literal[C.ENERGY, C.DISPLACEMENT] = DISPLACEMENT,
        softening: Literal[C.EXPONENTIAL, C.LINEAR, C.TABULAR, C.DISPLACEMENT] = LINEAR,
        useMixedMode: Boolean = OFF,
        mixedModeType: Literal[C.BK, C.ENERGY, C.POWER_LAW, C.TABULAR] = TABULAR,
        modeMixRatio: Literal[C.TRACTION, C.ENERGY, C.TABULAR] = ENERGY,
        exponent: Optional[float] = None,
        evolTempDep: Boolean = OFF,
        evolDependencies: int = 0,
        evolTable: tuple = (),
        useStabilization: Boolean = OFF,
        viscosityCoef: Optional[float] = None,
    ):
        """This method creates a ContactDamage object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        initTable
            A sequence of sequences of Floats specifying the values defining the damage initiation.
            The items in the table data are described below.
        criterion
            A SymbolicConstant specifying the type of data used to define the initiation of damage.
            Possible values are MAX_STRESS, MAX_SEPARATION, QUAD_TRACTION, and QUAD_SEPARATION. The
            default value is MAX_STRESS.
        initTempDep
            A Boolean specifying whether the initiation data depend on temperature. The default
            value is OFF.
        initDependencies
            An Int specifying the number of initiation data field variables. The default value is 0.
        useEvolution
            A Boolean specifying whether evolution data will be defined. The default value is OFF.
        evolutionType
            A SymbolicConstant specifying the type of data used to define the evolution of damage.
            This argument is valid only when **useEvolution** = ON. Possible values are DISPLACEMENT and
            ENERGY. The default value is DISPLACEMENT.
        softening
            A SymbolicConstant specifying the type of data used to define the evolution softening
            response. This argument is valid only when **useEvolution** = ON. The TABULAR value can be
            used only when **evolutionType** = DISPLACEMENT. Possible values are LINEAR, EXPONENTIAL,
            and TABULAR. The default value is LINEAR.
        useMixedMode
            A Boolean specifying whether evolution data be defined using dependent behavior modes.
            This argument is valid only when **useEvolution** = ON. The default value is OFF.
        mixedModeType
            A SymbolicConstant specifying the mode mix fracture criterion. This argument is valid
            only when **useEvolution** = ON and when **useMixedMode** = ON. The POWER_LAW and BK values can
            be used only when **evolutionType** = ENERGY. Possible values are TABULAR, POWER_LAW, and
            BK. The default value is TABULAR.
        modeMixRatio
            A SymbolicConstant specifying the mode mix ratio type. This argument is valid only when
            **useEvolution** = ON and when **useMixedMode** = ON. The TRACTION value can be used only when
            **mixedModeType** = TABULAR. Possible values are ENERGY and TRACTION. The default value is
            ENERGY.
        exponent
            None or a Float specifying the exponent in the power-law or BK criterion that defines
            the variation of fracture energy with mode mix. This argument is valid only when
            **useEvolution** = ON and when **mixedModeType** = POWER_LAW or BK. The default value is None.
        evolTempDep
            A Boolean specifying whether the evolution data depend on temperature. This argument is
            valid only when **useEvolution** = ON. The default value is OFF.
        evolDependencies
            An Int specifying the number of evolution data field variables. This argument is valid
            only when **useEvolution** = ON. The default value is 0.
        evolTable
            A sequence of sequences of Floats specifying the values defining the damage evolution.
            The items in the table data are described below. This argument is valid only when
            **useEvolution** = ON.
        useStabilization
            A Boolean specifying whether stabilization data will be defined. This argument is valid
            only when **useEvolution** = ON. The default value is OFF.
        viscosityCoef
            None or a Float specifying the viscosity coefficient. This argument is valid only when
            **useStabilization** = ON. The default value is None.

        Returns
        -------
        ContactDamage
            A ContactDamage object.
        """
        self.damage = ContactDamage(
            initTable,
            criterion,
            initTempDep,
            initDependencies,
            useEvolution,
            evolutionType,
            softening,
            useMixedMode,
            mixedModeType,
            modeMixRatio,
            exponent,
            evolTempDep,
            evolDependencies,
            evolTable,
            useStabilization,
            viscosityCoef,
        )
        return self.damage

    @abaqus_method_doc
    def FractureCriterion(
        self,
        initTable: tuple,
        type: Literal[C.ENHANCED, C.VCCT] = VCCT,
        mixedModeBehavior: Literal[C.REEDER, C.BK, C.POWER] = BK,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        tolerance: float = 0,
        specifyUnstableCrackProp: Union[SymbolicConstant, Boolean] = OFF,
        unstableTolerance: Union[Literal[C.DEFAULT], float] = DEFAULT,
    ):
        """This method creates a FractureCriterion object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        initTable
            A sequence of sequences of Floats specifying the value defining the fracture criterion.
            The items in the table data are described below.
        type
            A SymbolicConstant specifying the type of data used to define the fracture criterion.
            Possible values are VCCT and ENHANCED VCCT. The default value is VCCT.
        mixedModeBehavior
            A SymbolicConstant specifying the mixed mode behavior type used to define fracture
            criterion. Possible values are BK, POWER, and REEDER. The default value is BK.
        temperatureDependency
            A Boolean specifying whether the fracture criterion data depend on temperature. The
            default value is OFF.
        dependencies
            An Int specifying the number of fracture criterion data field variables. The default
            value is 0.
        tolerance
            A Float specifying the tolerance for VCCT Enhanced VCCT type. The default value is 0.2.
        specifyUnstableCrackProp
            A SymbolicConstant specifying whether to include unstable crack growth tolerance in
            fracture criterion. Possible values are ON and OFF. The default value is OFF.
        unstableTolerance
            The SymbolicConstant DEFAULT or a Float specifying the tolerance for unstable crack
            propagation. This parameter specified only if **specifyUnstableCrackProp** = ON. The default
            value is DEFAULT.

        Returns
        -------
        FractureCriterion
            A FractureCriterion object.
        """
        self.fractureCriterion = FractureCriterion(
            initTable,
            type,
            mixedModeBehavior,
            temperatureDependency,
            dependencies,
            tolerance,
            specifyUnstableCrackProp,
            unstableTolerance,
        )
        return self.fractureCriterion

    @abaqus_method_doc
    def CohesiveBehavior(
        self,
        repeatedContacts: Boolean = OFF,
        eligibility: Literal[C.INITIAL_NODES, C.SPECIFIED, C.ALL_NODES] = ALL_NODES,
        defaultPenalties: Boolean = ON,
        coupling: Literal[C.COUPLED, C.UNCOUPLED] = UNCOUPLED,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        table: tuple = (),
    ):
        """This method creates a CohesiveBehavior object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        repeatedContacts
            A Boolean specifying whether to enforce cohesive behavior for recurrent contacts at
            nodes on the secondary surface subsequent to ultimate failure. The default value is OFF.
        eligibility
            A SymbolicConstant specifying the eligible secondary nodes. Possible values are
            ALL_NODES, INITIAL_NODES, and SPECIFIED. The default value is ALL_NODES.
        defaultPenalties
            A Boolean specifying whether to use the default contact penalties. The default value is
            ON.
        coupling
            A SymbolicConstant specifying whether the traction-separation coefficients are coupled
            or uncoupled. This argument is valid only for **defaultPenalties** = OFF. Possible values
            are UNCOUPLED and COUPLED. The default value is UNCOUPLED.
        temperatureDependency
            A Boolean specifying whether the coefficient data depend on temperature. This argument
            is valid only for **defaultPenalties** = OFF. The default value is OFF.
        dependencies
            An Int specifying the number of field variables. This argument is valid only for
            **defaultPenalties** = OFF. The default value is 0.
        table
            A sequence of sequences of Floats specifying the traction-separation coefficients. The
            items in the table data are described below. This argument is valid only for
            **defaultPenalties** = OFF.

        Returns
        -------
        CohesiveBehavior
            A CohesiveBehavior object.
        """
        self.cohesiveBehavior = CohesiveBehavior(
            repeatedContacts,
            eligibility,
            defaultPenalties,
            coupling,
            temperatureDependency,
            dependencies,
            table,
        )
        return self.cohesiveBehavior

    @abaqus_method_doc
    def ThermalConductance(
        self,
        definition: Literal[C.USER_DEFINED, C.TABULAR] = TABULAR,
        clearanceDependency: Boolean = ON,
        pressureDependency: Boolean = OFF,
        temperatureDependencyC: Boolean = OFF,
        massFlowRateDependencyC: Boolean = OFF,
        dependenciesC: int = 0,
        clearanceDepTable: tuple = (),
        temperatureDependencyP: Boolean = OFF,
        massFlowRateDependencyP: Boolean = OFF,
        dependenciesP: int = 0,
        pressureDepTable: tuple = (),
    ):
        """This method creates a ThermalConductance object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        definition
            A SymbolicConstant specifying how the thermal conductance is defined. Possible values
            are TABULAR and USER_DEFINED. The default value is TABULAR.
        clearanceDependency
            A Boolean specifying whether to use clearance-dependent data. The default value is ON.
        pressureDependency
            A Boolean specifying whether to use pressure-dependent data. The default value is OFF.
        temperatureDependencyC
            A Boolean specifying whether to use temperature-dependent data with clearance
            dependency. The default value is OFF.
        massFlowRateDependencyC
            A Boolean specifying whether to use mass-flow-rate-dependent data with clearance
            dependency. The default value is OFF.
        dependenciesC
            An Int specifying the number of field variables to use with clearance dependency. The
            default value is 0.
        clearanceDepTable
            A sequence of sequences of Floats specifying clearance dependency data. The items in the
            table data are described below.
        temperatureDependencyP
            A Boolean specifying whether to use temperature-dependent data with pressure dependency.
            The default value is OFF.
        massFlowRateDependencyP
            A Boolean specifying whether to use mass-flow-rate-dependent data with pressure
            dependency. The default value is OFF.
        dependenciesP
            An Int specifying the number of field variables to use with pressure dependency. The
            default value is 0.
        pressureDepTable
            A sequence of sequences of Floats specifying pressure dependency data. The items in the
            table data are described below.

        Returns
        -------
        ThermalConductance
            A ThermalConductance object.
        """
        self.thermalConductance = ThermalConductance(
            definition,
            clearanceDependency,
            pressureDependency,
            temperatureDependencyC,
            massFlowRateDependencyC,
            dependenciesC,
            clearanceDepTable,
            temperatureDependencyP,
            massFlowRateDependencyP,
            dependenciesP,
            pressureDepTable,
        )
        return self.thermalConductance

    @abaqus_method_doc
    def HeatGeneration(self, conversionFraction: float = 1, secondaryFraction: float = 0):
        """This method creates a GapHeatGeneration object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        conversionFraction
            A Float specifying the fraction of dissipated energy caused by friction or electric
            currents that is converted to heat. The default value is 1.0.
        secondaryFraction
            A Float specifying the fraction of converted heat distributed to the secondary surface.
            The default value is 0.5.

            .. versionchanged:: 2022
                The argument `slaveFraction` was renamed to `secondaryFraction`.

        Returns
        -------
        GapHeatGeneration
            A GapHeatGeneration object.
        """
        self.heatGeneration = GapHeatGeneration(conversionFraction, secondaryFraction)
        return self.heatGeneration

    @abaqus_method_doc
    def Radiation(self, mainEmissivity: float, secondaryEmissivity: float, table: tuple):
        """This method creates a Radiation object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        mainEmissivity
            A Float specifying the emissivity of the main surface.

            .. versionchanged:: 2022
                The argument `masterEmissivity` was renamed to `mainEmissivity`.
        secondaryEmissivity
            A Float specifying the emissivity of the secondary surface.

            .. versionchanged:: 2022
                The argument `slaveEmissivity` was renamed to `secondaryEmissivity`.
        table
            A sequence of sequences of Floats specifying the following:Effective viewfactor, FF.Gap
            clearance, dd.

        Returns
        -------
        Radiation
            A Radiation object.
        """
        self.radiation = Radiation(mainEmissivity, secondaryEmissivity, table)
        return self.radiation

    @abaqus_method_doc
    def GeometricProperties(
        self,
        contactArea: float = 1,
        padThickness: Optional[float] = None,
        trackingThickness: Optional[float] = None,
        dependentVariables: int = 0,
        numProperties: int = 0,
        useUnsymmetricEqunProcedure: Boolean = OFF,
        modelType: Optional[SymbolicConstant] = None,
    ):
        """This method creates a GeometricProperties object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        contactArea
            A Float specifying the out-of-plane thickness of the surface for a two-dimensional model
            or cross-sectional area for every node in the node-based surface. The default value is
            1.0.
        padThickness
            None or a Float specifying the thickness of an interfacial layer between the contacting
            surfaces. If **padThickness** = None, there is no interfacial layer. The default value is
            None.
        trackingThickness
            None or a Float specifying the thickness that determines the contacting surfaces to be
            tracked. The input value for this parameter cannot be negative. An internal default
            value is used if a zero value is input or if the parameter is omitted.
        dependentVariables
            An Int specifying the number of state-dependent variables. The default value is 0. This
            argument is applicable only if **modelType** = MODELTYPE_USER or
            **modelType** = MODELTYPE_USER_INTERACTION.
        numProperties
            An Int specifying the number of property values required. The default value is 0. This
            argument is applicable only if **modelType** = MODELTYPE_USER or
            **modelType** = MODELTYPE_USER_INTERACTION.
        useUnsymmetricEqunProcedure
            A Boolean specifying whether to use unsymmetric equation solution procedures. This
            argument is applicable only if **modelType** = MODELTYPE_USER or
            **modelType** = MODELTYPE_USER_INTERACTION.
        modelType
            A SymbolicConstant specifying the surface interaction model type.

        Returns
        -------
        GeometricProperties
            A GeometricProperties object.
        """
        self.geometricProperties = GeometricProperties(
            contactArea,
            padThickness,
            trackingThickness,
            dependentVariables,
            numProperties,
            useUnsymmetricEqunProcedure,
            modelType,
        )
        return self.geometricProperties

    @abaqus_method_doc
    def ElectricalConductance(
        self,
        definition: Literal[C.USER_DEFINED, C.TABULAR] = TABULAR,
        clearanceDependency: Boolean = ON,
        pressureDependency: Boolean = OFF,
        temperatureDependencyC: Boolean = OFF,
        dependenciesC: int = 0,
        clearanceDepTable: tuple = (),
        temperatureDependencyP: Boolean = OFF,
        dependenciesP: int = 0,
        pressureDepTable: tuple = (),
    ):
        """This method creates a GapElectricalConductance object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        definition
            A SymbolicConstant specifying how the electrical conductance is defined. Possible values
            are TABULAR and USER_DEFINED. The default value is TABULAR.
        clearanceDependency
            A Boolean specifying whether to use clearance-dependent data. The default value is ON.
        pressureDependency
            A Boolean specifying whether to use pressure-dependent data. The default value is OFF.
        temperatureDependencyC
            A Boolean specifying whether to use temperature-dependent data with clearance
            dependency. The default value is OFF.
        dependenciesC
            An Int specifying the number of field variables to use with clearance dependency. The
            default value is 0.
        clearanceDepTable
            A sequence of sequences of Floats specifying clearance dependency data. The items in the
            table data are described below.
        temperatureDependencyP
            A Boolean specifying whether to use temperature-dependent data with pressure dependency.
            The default value is OFF.
        dependenciesP
            An Int specifying the number of field variables to use with pressure dependency. The
            default value is 0.
        pressureDepTable
            A sequence of sequences of Floats specifying pressure dependency data. The items in the
            table data are described below.

        Returns
        -------
        GapElectricalConductance
            A GapElectricalConductance object.
        """
        self.electricalConductance = GapElectricalConductance(
            definition,
            clearanceDependency,
            pressureDependency,
            temperatureDependencyC,
            dependenciesC,
            clearanceDepTable,
            temperatureDependencyP,
            dependenciesP,
            pressureDepTable,
        )
        return self.electricalConductance



from typing import Union, Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal

from .BoundaryCondition import BoundaryCondition
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import SET, SymbolicConstant, UNIFORM, UNSET
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class VelocityBC(BoundaryCondition):
    """The VelocityBC object stores the data for a velocity boundary condition.
    The VelocityBC object is derived from the BoundaryCondition object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].boundaryConditions[name]
    """

    #: A String specifying the boundary condition repository key.
    name: str = ""

    #: A SymbolicConstant specifying how the boundary condition is distributed spatially.
    #: Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.
    distributionType: SymbolicConstant = UNIFORM

    #: A String specifying the name of the AnalyticalField object associated with this boundary
    #: condition. The **fieldName** argument applies only when **distributionType** = FIELD. The
    #: default value is an empty string.
    fieldName: str = ""

    #: A SymbolicConstant specifying the category of the boundary condition. Possible values
    #: are MECHANICAL and THERMAL.
    category: Optional[SymbolicConstant] = None

    #: A Region object specifying the region to which the boundary condition is applied.
    region: Region = Region()

    #: None or a DatumCsys object specifying the local coordinate system of the boundary
    #: condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
    #: in the global coordinate system. The default value is None.
    localCsys: Optional[str] = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        fieldName: str = "",
        v1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        v2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        v3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        vr1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        vr2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        vr3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        amplitude: str = UNSET,
        localCsys: Optional[str] = None,
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM] = UNIFORM,
    ):
        """This method creates a VelocityBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].VelocityBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        fieldName
            A String specifying the name of the AnalyticalField object associated with this boundary
            condition. The **fieldName** argument applies only when **distributionType** = FIELD. The
            default value is an empty string.
        v1
            A Float or a SymbolicConstant specifying the velocity component in the 1-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is
            UNSET. Note: Although **v1**, **v2**, **v3**, **vr1**, **vr2**, and **vr3** are optional arguments, at
            least one of them must be specified.
        v2
            A Float or a SymbolicConstant specifying the velocity component in the 2-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        v3
            A Float or a SymbolicConstant specifying the velocity component in the 3-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        vr1
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        vr2
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        vr3
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.

        Returns
        -------
        VelocityBC
            A VelocityBC object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        fieldName: str = "",
        v1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        v2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        v3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        vr1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        vr2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        vr3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        amplitude: str = UNSET,
        localCsys: Optional[str] = None,
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM] = UNIFORM,
    ):
        """This method modifies the data for an existing VelocityBC object in the step where it is
        created.

        Parameters
        ----------
        fieldName
            A String specifying the name of the AnalyticalField object associated with this boundary
            condition. The **fieldName** argument applies only when **distributionType** = FIELD. The
            default value is an empty string.
        v1
            A Float or a SymbolicConstant specifying the velocity component in the 1-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is
            UNSET. Note: Although **v1**, **v2**, **v3**, **vr1**, **vr2**, and **vr3** are optional arguments, at
            least one of them must be specified.
        v2
            A Float or a SymbolicConstant specifying the velocity component in the 2-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        v3
            A Float or a SymbolicConstant specifying the velocity component in the 3-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        vr1
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        vr2
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        vr3
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(
        self,
        stepName: str,
        v1: Union[Literal[C.SET, C.FREED], float] = SET,
        v2: Union[Literal[C.SET, C.FREED], float] = SET,
        v3: Union[Literal[C.SET, C.FREED], float] = SET,
        vr1: Union[Literal[C.SET, C.FREED], float] = SET,
        vr2: Union[Literal[C.SET, C.FREED], float] = SET,
        vr3: Union[Literal[C.SET, C.FREED], float] = SET,
        amplitude: str = "",
    ):
        """This method modifies the propagating data for an existing VelocityBC object in the
        specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the boundary condition is modified.
        v1
            A Float or a SymbolicConstant specifying the velocity component in the 1-direction.
            Possible values for the SymbolicConstant are SET and FREED.
        v2
            A Float or a SymbolicConstant specifying the velocity component in the 2-direction.
            Possible values for the SymbolicConstant are SET and FREED.
        v3
            A Float or a SymbolicConstant specifying the velocity component in the 3-direction.
            Possible values for the SymbolicConstant are SET and FREED.
        vr1
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            1-direction. Possible values for the SymbolicConstant are SET and FREED.
        vr2
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            2-direction. Possible values for the SymbolicConstant are SET and FREED.
        vr3
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            3-direction. Possible values for the SymbolicConstant are SET and FREED.
        amplitude
            A String or a SymbolicConstant specifying the name of the amplitude reference. Possible
            values for the SymbolicConstant are UNCHANGED and FREED. UNCHANGED should be used if the
            amplitude is propagated from the previous analysis step. FREED should be used if the
            boundary condition is changed to have no amplitude reference. You should provide the
            **amplitude** argument only if it is valid for the specified step.
        """
        ...



from abaqus.Connector.ConnectorDamage import ConnectorDamage  # noqa # pylint: disable=unused-import
from abaqus.Connector.ConnectorDamping import ConnectorDamping  # noqa # pylint: disable=unused-import
from abaqus.Connector.ConnectorElasticity import ConnectorElasticity  # noqa # pylint: disable=unused-import
from abaqus.Connector.ConnectorFailure import ConnectorFailure  # noqa # pylint: disable=unused-import
from abaqus.Connector.ConnectorFriction import ConnectorFriction  # noqa # pylint: disable=unused-import
from abaqus.Connector.ConnectorLock import ConnectorLock  # noqa # pylint: disable=unused-import
from abaqus.Connector.ConnectorPlasticity import ConnectorPlasticity  # noqa # pylint: disable=unused-import
from abaqus.Connector.ConnectorPotential import ConnectorPotential  # noqa # pylint: disable=unused-import
from abaqus.Connector.ConnectorStop import ConnectorStop  # noqa # pylint: disable=unused-import



"""
Compression Model
=================

A simple example to establish a compression model.

.. figure:: /images/model.png
    :width: 50%
    :align: center

    A simple compression model.
"""

from abaqus import *
from abaqusConstants import *
from caeModules import *
from driverUtils import *

executeOnCaeStartup()

# Model
model = mdb.models['Model-1']

# Part
sketch = model.ConstrainedSketch(name='sketch', sheetSize=1.0)
sketch.rectangle((0, 0), (1, 1))
part = model.Part(name='part', dimensionality=THREE_D, type=DEFORMABLE_BODY)
part.BaseSolidExtrude(sketch=sketch, depth=1)

# Create sets
part.Set(name='set-all', cells=part.cells.findAt(coordinates=((0.5, 0.5, 0.5),)))
part.Set(name='set-bottom', faces=part.faces.findAt(coordinates=((0.5, 0.5, 0.0),)))
part.Set(name='set-top', faces=part.faces.findAt(coordinates=((0.5, 0.5, 1.0),)))
part.Surface(name='surface-top',
             side1Faces=part.faces.findAt(coordinates=((0.5, 0.5, 1.0),)))

# Assembly
model.rootAssembly.DatumCsysByDefault(CARTESIAN)
model.rootAssembly.Instance(name='instance', part=part, dependent=ON)

# Material
material = model.Material(name='material')
material.Elastic(table=((1000, 0.2),))
material.Density(table=((2500,),))

# Section
model.HomogeneousSolidSection(name='section', material='material', thickness=None)
part.SectionAssignment(region=part.sets['set-all'], sectionName='section')

# Step
step = model.StaticStep(name='Step-1', previous='Initial', description='',
                        timePeriod=1.0, timeIncrementationMethod=AUTOMATIC,
                        maxNumInc=100, initialInc=0.01, minInc=0.001, maxInc=0.1)

# Output request
field = model.FieldOutputRequest('F-Output-1', createStepName='Step-1',
                                 variables=('S', 'E', 'U'))

# Boundary condition
bottom_instance = model.rootAssembly.instances['instance'].sets['set-bottom']
bc = model.DisplacementBC(name='BC-1', createStepName='Initial',
                          region=bottom_instance, u1=SET, u2=SET, u3=SET, ur1=SET, ur2=SET, ur3=SET)

# Load
top_instance = model.rootAssembly.instances['instance'].surfaces['surface-top']
pressure = model.Pressure('pressure', createStepName='Step-1', region=top_instance,
                          magnitude=100)

# Mesh
elem1 = mesh.ElemType(elemCode=C3D8R)
elem2 = mesh.ElemType(elemCode=C3D6)
elem3 = mesh.ElemType(elemCode=C3D4)
part.setElementType(regions=(part.cells,), elemTypes=(elem1, elem2, elem3))
part.seedPart(size=0.1)
part.generateMesh()

# Job
job = mdb.Job(name='Job-1', model='Model-1')
job.writeInput()

# Submit the job
job.submit()
job.waitForCompletion()

# Save abaqus model
mdb.saveAs('compression.cae')



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class EventSeriesType:
    """The EventSeriesType object is used to define a type of event in a process.

    .. note::
        This object can be accessed by::

            mdb.models[name].eventSeriesTypes[name]

        The corresponding analysis keywords are:

        - EVENT SERIES TYPE
        - EVENT SERIES

    .. versionadded:: 2020
        The `EventSeriesType` class was added.
    """

    #: A String specifying the repository key.
    name: str

    #: A string specifying the step name.
    createStepName: str

    #: A String array specifying fields. The default value is an empty array.
    fields: str = ""

    @abaqus_method_doc
    def __init__(self, name: str, createStepName: str, fields: str = ""):
        """This method creates an EventSeriesType object.

        .. note::
            This function can be accessed by::

                mdb.models[name].EventSeriesType

        Parameters
        ----------
        name
            A String specifying the repository key.
        createStepName
            A string specifying the step name.
        fields
            A String array specifying fields. The default value is an empty array.

        Returns
        -------
        EventSeriesType
            A EventSeriesType object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, fields: str = ""):
        """This method modifies the EventSeriesType object.

        Parameters
        ----------
        fields
            A String array specifying fields. The default value is an empty array.

        Returns
        -------

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from typing import overload, Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .AreaStyle import AreaStyle
from .LineStyle import LineStyle
from .QuantityType import QuantityType
from .SymbolStyle import SymbolStyle
from .TextStyle import TextStyle
from .XYData import XYData
from .XYPlotBase import XYPlotBase
from ..UtilityAndView.abaqusConstants import Boolean, FILLED_CIRCLE, ON, SOLID, SymbolicConstant


@abaqus_class_doc
class XYPlot(XYPlotBase):
    @staticmethod
    @abaqus_method_doc
    def AreaStyle(color: str = "", fill: Boolean = ON, style: SymbolicConstant = SOLID) -> AreaStyle:
        """This method creates an AreaStyle.

        .. note::
            This function can be accessed by::

                session.AreaStyle
                xyPlot.AreaStyle

        Parameters
        ----------
        color
            A String specifying the color to be used when filling an area with this AreaStyle
            object. The default value is "White".
        fill
            A Boolean specifying whether to fill the area when using this AreaStyle. The default
            value is ON.
        style
            A SymbolicConstant specifying the area pattern style to be used when filling an area
            using this AreaStyle. The default value is SOLID.

        Returns
        -------
        AreaStyle
            An AreaStyle object.

        Raises
        ------
        ColorError
        """
        areaStyle = AreaStyle(color, fill, style)
        return areaStyle

    @staticmethod
    @abaqus_method_doc
    def LineStyle(
        color: str = "",
        show: Boolean = ON,
        style: SymbolicConstant = SOLID,
        thickness: float = 0,
    ) -> LineStyle:
        """This method creates a LineStyle.

        .. note::
            This function can be accessed by::

                session.LineStyle
                xyPlot.LineStyle

        Parameters
        ----------
        color
            A String specifying the color to be used when drawing a line with this LineStyle object.
            The default value is "White".
        show
            A Boolean specifying whether to draw the line when using this LineStyle. The default
            value is ON.
        style
            A SymbolicConstant specifying the line style to be used when drawing lines using this
            LineStyle. Possible values are SOLID, DASHED, DOTTED, and DOT_DASH. The default value is
            SOLID.
        thickness
            A Float specifying the line thickness in mm to be used when drawing lines using this
            LineStyle. The default value is 0.2.

        Returns
        -------
        LineStyle
            A LineStyle object.

        Raises
        ------
        ColorError
        """
        lineStyle = LineStyle(color, show, style, thickness)
        return lineStyle

    @staticmethod
    @abaqus_method_doc
    def QuantityType(label: str = "", type: Optional[SymbolicConstant] = None) -> QuantityType:
        """This method creates a QuantityType object.

        .. note::
            This function can be accessed by::

                session.QuantityType
                xyPlot.QuantityType

        Parameters
        ----------
        label
            A String specifying the label for this quantity type.
        type
            A SymbolicConstant specifying the physical dimension of the axis. Possible values are:

            - NONE.
            - ACCELERATION.
            - ACOUSTIC_INTENSITY.
            - ANGLE.
            - ANGULAR_MOMENTUM.
            - ARC_LENGTH.
            - AREA.
            - AREA_VELOCITY_SQUARED, specifying "Velocity squared per area".
            - BIMOMENT.
            - CURVATURE.
            - CORIOLIS_LOAD.
            - DAMAGE.
            - DAMAGE_CRITERION.
            - DENSITY.
            - DENSITY_ROTATIONAL_ACCELERATION, specifying "Density * Angular acceleration".
            - DISPLACEMENT.
            - ECURRENT_AREA_TIME, specifying "Time integrated electric current per area".
            - ELECTRIC_CHARGE.
            - ELECTRIC_CURRENT.
            - ELECTRIC_CURRENT_AREA, specifying "Electric current per unit area".
            - ELECTRIC_POTENTIAL.
            - ENERGY.
            - ENERGY_DENSITY.
            - ENERGY_RELEASE_RATE.
            - EPOTENTIAL_GRADIENT, specifying "Electric potential gradient".
            - FREQUENCY.
            - FORCE.
            - FORCE_VOLUME, specifying "Force per volume".
            - HEAT_FLUX.
            - HEAT_FLUX_AREA, specifying "Heat flux per area".
            - HEAT_FLUX_RATE.
            - HEAT_FLUX_VOLUME, specifying "Heat flux per volume".
            - LENGTH.
            - LINEAR_PRESSURE.
            - LUMIN, specifying "Luminous intensity".
            - MASS.
            - MASS_FLOW_AREA, specifying "Mass flow per area".
            - MASS_FLOW_AREA_RATE, specifying "Mass flow rate per area".
            - MASS_FLOW_RATE.
            - MODE_NUMBER.
            - MOMENT.
            - NUMBER.
            - PATH.
            - PHASE.
            - POSITION.
            - PRESSURE.
            - PRESSURE_GRADIENT.
            - RATE.
            - ROTARY_INERTIA.
            - ROTATIONAL_ACCELERATION.
            - ROTATIONAL_VELOCITY.
            - STATUS.
            - STRAIN.
            - STRAIN_RATE.
            - STRESS.
            - STRESS_INTENS_FACTOR, specifying "Stress intensity factor".
            - SUBSTANCE, specifying "Amount of substance".
            - TEMPERATURE.
            - THICKNESS.
            - TIME.
            - TIME_INCREMENT.
            - TIME_HEAT_FLUX, specifying "Time integrated heat flux".
            - TIME_HEAT_FLUX_AREA, specifying "Time integrated heat flux per area".
            - TIME_VOLUME, specifying "Time integrated volume".
            - TIME_VOLUME_FLUX, specifying "Time integrated volume flux per area".
            - TWIST.
            - VELOCITY.
            - VELOCITY_SQUARED.
            - VOLUME.
            - VOLUME_FLUX.
            - VOLUME_FLUX_AREA, specifying "Volume flux per area".
            - VOLUME_FRACTION.

            The default value is NONE

        Returns
        -------
        QuantityType
            A QuantityType object.
        """
        quantityType = QuantityType(label, type)
        return quantityType

    @staticmethod
    @abaqus_method_doc
    def SymbolStyle(
        color: str = "",
        show: Boolean = ON,
        marker: SymbolicConstant = FILLED_CIRCLE,
        size: float = 2,
    ) -> SymbolStyle:
        """This method creates a SymbolStyle object.

        .. note::
            This function can be accessed by::

                session.SymbolStyle
                xyPlot.SymbolStyle

        Parameters
        ----------
        color
            A String specifying the color to be used when drawing a marker with this SymbolStyle
            object. The default value is "White".
        show
            A Boolean specifying whether to draw the marker when using this SymbolStyle object. The
            default value is ON.
        marker
            A SymbolicConstant specifying the marker type be used when drawing symbols using this
            SymbolStyle object. Possible values are:

            - FILLED_CIRCLE
            - FILLED_SQUARE
            - FILLED_DIAMOND
            - FILLED_TRI
            - HOLLOW_CIRCLE
            - HOLLOW_SQUARE
            - HOLLOW_DIAMOND
            - HOLLOW_TRI
            - CROSS
            - XMARKER
            - POINT

            The default value is FILLED_CIRCLE.
        size
            A Float specifying the marker size to be used when drawing markers using this
            SymbolStyle object. The default value is 2.0.

        Returns
        -------
        SymbolStyle
            A SymbolStyle object.

        Raises
        ------
        ColorError
        """
        symbolStyle = SymbolStyle(color, show, marker, size)
        return symbolStyle

    @staticmethod
    @abaqus_method_doc
    def TextStyle(color: str = "", show: Boolean = ON, font: str = "", rotationAngle: float = 0) -> TextStyle:
        """This method creates a TextStyle.

        .. note::
            This function can be accessed by::

                session.TextStyle
                xyPlot.TextStyle

        Parameters
        ----------
        color
            A String specifying the color to be used when drawing text with this TextStyle object.
            The default value is "White".
        show
            A Boolean specifying whether to draw the text when using this TextStyle object. The
            default value is ON.
        font
            A String specifying the name of the font to be used when drawing text with this
            TextStyle object. The default value is "-*-verdana-medium-r-normal-*-*-120-*-*-p-*-*-*".
        rotationAngle
            A Float specifying the angle in degrees used for displaying the text. The default value
            is 0.0.

        Returns
        -------
        TextStyle
            A TextStyle object.

        Raises
        ------
        ColorError
        """
        textStyle = TextStyle(color, show, font, rotationAngle)
        return textStyle

    @staticmethod
    @overload
    @abaqus_method_doc
    def XYData(
        data: tuple,
        name: str = "",
        sourceDescription: str = "",
        contentDescription: str = "",
        positionDescription: str = "",
        legendLabel: str = "",
        xValuesLabel: str = "",
        yValuesLabel: str = "",
        axis1QuantityType: Optional[QuantityType] = None,
        axis2QuantityType: Optional[QuantityType] = None,
    ) -> XYData:
        """This method creates an XYData object from a sequence of **X - Y** data pairs.

        .. note::
            This function can be accessed by::

                session.XYData
                xyPlot.XYData

        Parameters
        ----------
        data
            A sequence of pairs of Floats specifying the **X - Y** data pairs.
        name
            The repository key. If the name is not supplied while creating the XYData object using
            xyPlot.XYData, a default name in the form _temp#_ is generated and the XYData object is
            temporary. (This argument is required if the method is accessed from the session
            object.)
        sourceDescription
            A String specifying the source of the **X - Y** data (e.g., “Entered from keyboard”, “Taken
            from ASCII file”, “Read from an ODB”, etc.). The default value is an empty string.
        contentDescription
            A String specifying the content of the **X - Y** data (e.g., “field 1 vs. field 2”). The
            default value is an empty string.
        positionDescription
            A String specifying additional information about the **X - Y** data (e.g., “for whole
            model”). The default value is an empty string.
        legendLabel
            A String specifying the label to be used in the legend. The default value is the name of
            the XYData object.
        xValuesLabel
            A String specifying the label for the X-values. This value may be overridden if the
            **X - Y** data are combined with other **X - Y** data. The default value is an empty string.
        yValuesLabel
            A String specifying the label for the Y-values. This value may be overridden if the
            **X - Y** data are combined with other **X - Y** data. The default value is an empty string.
        axis1QuantityType
            A QuantityType object specifying the QuantityType object associated to the X -axis1-
            values.
        axis2QuantityType
            A QuantityType object specifying the QuantityType object associated to the Y -axis2-
            values.

        Returns
        -------
        XYData
            An XYData object.
        """
        ...

    @staticmethod
    @overload
    @abaqus_method_doc
    def XYData(objectToCopy: XYData) -> XYData:
        """This method creates an XYData object by copying an existing XYData object.

        .. note::
            This function can be accessed by::

                session.odbs[name].userData.XYData
                session.XYData
                xyPlot.XYData

        Parameters
        ----------
        objectToCopy
            An XYData object to be copied.

        Returns
        -------
        XYData
            An XYData object.
        """
        ...

    @staticmethod
    def XYData(*args, **kwargs) -> XYData:
        return XYData(())



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Calibration.Calibration import Calibration
from ..Model.ModelBase import ModelBase


@abaqus_class_doc
class CalibrationModel(ModelBase):
    """Abaqus creates a Model object named `Model-1` when a session is started.

    .. note::
        This object can be accessed by::

            mdb.models[name]
    """

    @abaqus_method_doc
    def Calibration(self, name: str) -> Calibration:
        """This method creates a Calibration object.

        .. note::
            This function can be accessed by::

                mdb.models[name].Calibration

        Parameters
        ----------
        name
            A String specifying the name of the new calibration.

        Returns
        -------
        Calibration
            A Calibration object.
        """
        self.calibrations[name] = calibration = Calibration(name)
        return calibration



from typing import Optional

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class SubstructureGenerateModes:
    """A SubstructureGenerateModes object is used to define the modes to be used in a modal
    dynamic analysis.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].steps[name].modeRange[i]
    """

    #: An Int specifying the mode number of the lowest mode of a range.
    start: Optional[int] = None

    #: An Int specifying the mode number of the highest mode of a range.
    end: Optional[int] = None

    #: An Int specifying the increment used to define the intermediate mode numbers beginning
    #: from the lowest mode to the highest mode.
    increment: Optional[int] = None



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .ConstrainedSketchConstraint import ConstrainedSketchConstraint
from ..ConstrainedSketchGeometry.ConstrainedSketchGeometry import (
    ConstrainedSketchGeometry,
)


@abaqus_class_doc
class ParallelConstraint(ConstrainedSketchConstraint):
    @abaqus_method_doc
    def __init__(self, entity1: ConstrainedSketchGeometry, entity2: ConstrainedSketchGeometry):
        """This method creates a parallel constraint. This constraint applies to lines and
        constrains them to be parallel.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].ParallelConstraint

        Parameters
        ----------
        entity1
            A ConstrainedSketchGeometry object specifying the first line.
        entity2
            A ConstrainedSketchGeometry object specifying the second line.

        Returns
        -------
        ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object.

        """
        ...



from typing import List

from .StructuralDampingByFrequencyComponent import StructuralDampingByFrequencyComponent

StructuralDampingByFrequencyComponentArray = List[StructuralDampingByFrequencyComponent]



import functools
from importlib import metadata

from docutils import nodes
from sphinx.application import Sphinx


def version_role(name, rawtext, text, lineno, inliner, options=None, content=None):
    """Role for version numbers.
    
    Parameters
    ----------
    name : str
        The local name of the interpreted role, the role name actually used in the document.
    rawtext : str
        A string containing the entire interpreted text input, including the role and markup. 
        Return it as a problematic node linked to a system message if a problem is encountered.
    text : str
        The interpreted text content.
    lineno : int
        The line number where the text block containing the interpreted text begins.
    inliner : `docutils.parsers.rst.states.Inliner`
        The docutils.parsers.rst.states.Inliner object that called role_fn. It contains the 
        several attributes useful for error reporting and document tree access.
    options : dict
        A dictionary of directive options for customization (from the "role" directive), to 
        be interpreted by the role function. Used for additional attributes for the generated elements and other functionality.
    content : list
        A list of strings, the directive content for customization (from the "role" directive). 
        To be interpreted by the role function.
        
    Returns
    -------
    list[Node]
        A list of nodes which will be inserted into the document tree at the point where the 
        interpreted role was encountered (can be an empty list).
    list[str]
        A list of system messages, which will be inserted into the document tree immediately 
        after the end of the current block (can also be empty).
    """
    version = '.'.join(metadata.version(text).split('.')[:3])
    node = nodes.Text(f"{text}=={version}", f"{text}=={version}")
    return [node], []
    

def setup(app: Sphinx):
    app.add_role('autopkg', version_role)
    return {'version': '0.0.1', 'parallel_read_safe': True}
